@@ -1,4 +1,5 @@
 from decimal import Decimal
+from ipaddress import IPv4Address, IPv6Address, IPv4Network, IPv6Network
 from typing import Union, Coroutine, Awaitable, Any, TypeVar, Callable
 
 Number = Union[Decimal, int, float]
@@ -10,6 +11,13 @@ NumberStr = Union[Decimal, int, float, str]
 VAL_FUNC_CORO = Union[Any, callable, Coroutine, Awaitable]
 """Type alias for callable's, coroutine's and awaitable's"""
 
+IP_OR_STR = Union[str, IPv4Address, IPv6Address]
+"""Shorthand union which accepts :class:`.IPv4Address`, :class:`.IPv6Address` or :class:`str`"""
+NET_OR_STR = Union[str, IPv4Network, IPv6Network]
+"""Shorthand union which accepts :class:`.IPv4Network`, :class:`.IPv6Network` or :class:`str`"""
+IP_NET_OR_STR = Union[str, IPv4Address, IPv6Address, IPv4Network, IPv6Network]
+"""Combined :py:type:`.IP_OR_STR` + :py:type:`.NET_OR_STR`"""
+
 T = TypeVar('T')
 """Plain generic type variable for use in helper functions"""
 K = TypeVar('K')
